import React, { useState } from 'react';
import {
	Spin,
	Space,
	Button,
	Row,
	Modal,
	message,
	TreeSelect,
	Form,
} from 'antd';
import { useRequest } from 'ahooks';
import ProCard from '@ant-design/pro-card';
import { ExclamationCircleOutlined } from '@ant-design/icons';
import 'antd/lib/tree-select/style/css';

import ProForm, { ModalForm, ProFormText } from '@ant-design/pro-form';

const URL_SCHEMA = {
	get: '/v1/zone/list',
	delete: '/v1/zone/',
	put: '/v1/zone/',
	add: '/v1/zone',
};
const NOt_OPERTION_RECORD = '001';

interface ResAreaProps {
	zone_name: string;
	zone_code: string;
	childs: ResAreaProps[];
}

interface UseAreaProps {
	id: string;
	value: string;
	title: string;
	children: UseAreaProps[];
}

/**
 * @description 将数据转换成 tree-select 需要得
 * @param list
 * @returns
 */
const resolveResData = (list: ResAreaProps[] = []) => {
	return list.map((element) => {
		return {
			...element,
			title: element.zone_name,
			id: element.zone_code,
			value: element.zone_code,
			children: resolveResData(element.childs),
		};
	});
};
const initialData: { list: UseAreaProps[] } = { list: [] };

/**
 * @description 区域管理
 * @returns
 */
const areaPage: React.FC = () => {
	const [form] = Form.useForm();
	const [updatModalState, changeUpdateModalState] = useState<{
		visible: boolean;
		currentRecord?: UseAreaProps;
	}>({
		visible: false,
	});

	const { loading, data, run: requestList } = useRequest(
		() => {
			return {
				url: URL_SCHEMA.get,
			};
		},
		{
			initialData: initialData,
			formatResult: (res: { list: ResAreaProps[] }) => {
				return {
					...res,
					list: resolveResData(res.list),
				};
			},
		}
	);
	const { run: requestDelete } = useRequest(
		(areaCode) => {
			return {
				method: 'DELETE',
				url: URL_SCHEMA.delete + areaCode,
			};
		},
		{
			manual: true,
			onError: (error) => {
				message.error(error.message);
			},
		}
	);
	const { run: requestUpdate } = useRequest(
		(data) => {
			return {
				url: URL_SCHEMA.put,
				method: 'PUT',
				data: { ...data, zone_name: data.title },
			};
		},
		{
			manual: true,
			onError: (error) => {
				message.error(error.message);
			},
			onSuccess: () => {
				message.success('修改成功!!!');
				requestList();
				changeUpdateModalState({ visible: false });
			},
		}
	);
	const { run: requestAdd } = useRequest(
		(data) => {
			return {
				url: URL_SCHEMA.add,
				method: 'POST',
				data: { ...data, zone_name: data.title },
			};
		},
		{
			manual: true,
			onError: (error) => {
				message.error(error.message);
			},
			onSuccess: () => {
				requestList();
				message.success('新增区域成功!!!');
			},
		}
	);
	const renderItem = (list: UseAreaProps[]) => {
		const handleClickRemove = (record) => {
			Modal.confirm({
				title: '确认删除？',
				icon: <ExclamationCircleOutlined />,
				content: record.zone_name,
				okText: '确认',
				cancelText: '取消',
				okButtonProps: {
					danger: true,
				},
				onOk: () => {
					return requestDelete(record.zone_code);
				},
			});
		};
		return list.map((element) => {
			return (
				<Row key={element.id}>
					<ProCard
						title={element.title}
						headerBordered
						collapsible
						defaultCollapsed
						extra={
							NOt_OPERTION_RECORD === element.id ? null : (
								<Space>
									<Button
										size='small'
										onClick={(e) => {
											e.stopPropagation();
											changeUpdateModalState({
												visible: true,
												currentRecord: element,
											});
										}}>
										编辑
									</Button>
									<Button
										danger
										type='primary'
										size='small'
										onClick={(e) => {
											e.stopPropagation();
											handleClickRemove(element);
										}}>
										删除
									</Button>
								</Space>
							)
						}>
						{renderItem(element.children)}
					</ProCard>
				</Row>
			);
		});
	};
	const renderModalForm = () => {
		return (
			<ProForm.Group>
				<ProFormText
					width='md'
					name='title'
					required
					label='区域名称'
					placeholder='请输入区域名称'
				/>
				<Form.Item
					label='上级区域'
					style={{ width: 300 }}
					name='parent_zone_code'>
					<TreeSelect
						placeholder='请选择上级区域'
						treeData={data?.list}></TreeSelect>
				</Form.Item>
			</ProForm.Group>
		);
	};
	return (
		<Spin spinning={loading}>
			<ModalForm
				form={form}
				modalProps={{
					onCancel: () => {
						form.resetFields();
					},
				}}
				trigger={
					<Button type='primary' style={{ marginBottom: 30 }}>
						新建
					</Button>
				}
				title='新建区域'
				onFinish={requestAdd}>
				{renderModalForm()}
			</ModalForm>
			<div>{renderItem(data?.list)}</div>
			{updatModalState.currentRecord ? (
				<ModalForm
					modalProps={{
						onCancel: () => {
							changeUpdateModalState({ visible: false });
						},
					}}
					title='修改区域'
					visible={updatModalState.visible}
					initialValues={updatModalState.currentRecord}
					onFinish={(data) => {
						return requestUpdate({
							...data,
							zone_code: updatModalState.currentRecord?.id,
						});
					}}>
					{renderModalForm()}
				</ModalForm>
			) : null}
		</Spin>
	);
};
export default areaPage;
