import React, { useContext, useState } from 'react';
import {
	Route,
	BrowserRouter as Router,
	Switch,
	Redirect,
} from 'react-router-dom';
import { routeConfig, routeItemProps } from '../config/routes';

function getUserInfo() {
	const localUserInfo = sessionStorage.getItem('userinfo') || '';
	try {
		return JSON.parse(localUserInfo);
	} catch (err) {
		return undefined;
	}
}

// 设置context
const authContext = React.createContext<{
	user: any;
	signin: any;
	signout: any;
} | null>(null);

export const useAuth = () => {
	return useContext(authContext);
};

function useProvideAuth() {
	const [user, setUser] = useState(getUserInfo());
	const signin = (data, cb) => {
		setUser(data);
		cb();
	};

	const signout = (data, cb) => {
		setUser(null);
		cb();
	};

	return {
		user,
		signin,
		signout,
	};
}

function ProvideAuth({ children }) {
	const auth = useProvideAuth();
	return <authContext.Provider value={auth}>{children}</authContext.Provider>;
}

const PrivateRoute = ({ children, ...rest }) => {
	const auth = useAuth();
	return (
		<Route
			{...rest}
			render={({ location }) =>
				auth?.user ? (
					children
				) : (
					<Redirect
						to={{
							pathname: '/login',
							state: { from: location },
						}}
					/>
				)
			}
		/>
	);
};

/**
 * @description 遍历生成路由并处理验证权限
 * @param routes
 * @returns
 */
const resolveRouters = (routes: routeItemProps[] = []) => {
	if (!routes.length) {
		return null;
	}
	return (
		<Switch>
			{routes.map(({ component: Component, ...rest }) => {
				// 查看是否有子路由需要生成
				const withChildRoutesNode = resolveRouters(rest.routes);
				// 处理需要验证权限得路由
				if (rest.authority) {
					return (
						<PrivateRoute key={rest.path}>
							{Component ? (
								<Component {...rest}>{withChildRoutesNode}</Component>
							) : (
								withChildRoutesNode
							)}
						</PrivateRoute>
					);
				}
				if (rest.redirect) {
					return (
						<Redirect
							key={rest.path}
							from={rest.path}
							to={rest.redirect}></Redirect>
					);
				}
				return (
					<Route
						{...rest}
						key={rest.path}
						render={(props) => {
							return <Component {...props}>{withChildRoutesNode}</Component>;
						}}></Route>
				);
			})}
		</Switch>
	);
};
const routerIndex: React.FC = () => {
	return (
		<ProvideAuth>
			<Router>{resolveRouters(routeConfig)}</Router>
		</ProvideAuth>
	);
};
export default routerIndex;
